@@ -907,6 +907,35 @@ static int maxmtu2maxsdu(uint8_t mtu)
     return sdu;
 }
 
+int maxmtu2itfmtu(uint8_t mtu)
+{
+
+    int sdu;
+
+    switch (mtu) {
+    case MTU_1516:
+        sdu = 1500;
+        break;
+    case MTU_1580:  /* LANE2: MTU can be 1580 too (IEEE 802.1p/Q) */
+        sdu = 1500;
+        break;
+    case MTU_4544:
+        sdu = 4550;
+        break;
+    case MTU_9234:
+        sdu = 9180;
+        break;
+    case MTU_18190:
+        sdu = 18200;
+        break;
+    default:
+        sdu = 1500;
+        break;
+    }
+
+    return sdu;
+}
+
 /* Convert a type of connection (CONTROL_CONN, DATA_DIRECT and MCAST_CONN)
  * to a BLLI codepoint which depends on C2 LAN Type.
  */
