@@ -63,7 +63,7 @@ void sig_reset(int a)
 static void usage(const char *progname)
 {
   printf("Usage: %s [-c LECS_address | -s LES_address] [-e esi] [-n VLAN_name]"
-    " [-m mesg_mask] [-l listen_address] [-i interface_number]"
+    " [-m mesg_mask] [-l listen_address | selector] [-i interface_number]"
     " [-t 1516|1580|4544|9234|18190] [-1] [-2] [-p] [-F logfile]"
     " [-f Fore specific name]\n", progname);
 }
@@ -188,7 +188,7 @@ int main(int argc, char **argv)
         int lane_version = 0;              /* LANE2 */
         int max_frame_size = MTU_UNSPEC;
         int lecs_method = LECS_WELLKNOWN;
-        int poll_ret = 0, itf = 0;
+        int poll_ret = 0, itf = 0, selector = 0;
         struct sockaddr_atmsvc manual_atm_addr;
         struct sockaddr_atmsvc listen_addr;
         
@@ -264,6 +264,16 @@ int main(int argc, char **argv)
                         set_verbosity(NULL, DIAG_DEBUG);
                         break;
                 case 'l':
+			if (isdigit(optarg[0]) && strlen(optarg) < 4 &&
+			  sscanf(optarg, "%d", &selector) &&
+			  selector >=0 && selector <= 0xff) {
+				listen_addr.sas_addr.prv[ATM_ESA_LEN - 1] =
+				  (char) selector;
+				listen_addr.sas_family = AF_ATMSVC; 
+				diag(COMPONENT, DIAG_INFO, "Selector byte set "
+				  "to %d", selector);
+			}
+			else
                         if (text2atm(optarg, (struct sockaddr *)&listen_addr,
                                      sizeof(struct sockaddr_atmsvc), T2A_NAME) < 0) {
                                 diag(COMPONENT, DIAG_ERROR, "Invalid ATM listen address\n");
