@@ -1,6 +1,6 @@
 /* atm.h - Functions useful for ATM applications */
  
-/* Written 1995-1998 by Werner Almesberger, EPFL-LRC/ICA */
+/* Written 1995-1999 by Werner Almesberger, EPFL-LRC/ICA */
  
 
 #ifndef _ATM_H
@@ -12,16 +12,32 @@
 
 
 /*
- * For glibc. gcc won't complain unless we define something else than what
- * is in sys/socket.h
+ * For versions of glibc < 2.1
  */
 
+#ifndef AF_ATMPVC
 #define AF_ATMPVC	8
+#endif
+
+#ifndef AF_ATMSVC
 #define AF_ATMSVC	20
+#endif
+
+#ifndef PF_ATMPVC
 #define PF_ATMPVC	AF_ATMPVC
+#endif
+
+#ifndef PF_ATMSVC
 #define PF_ATMSVC	AF_ATMSVC
+#endif
+
+#ifndef SOL_ATM
 #define SOL_ATM		264
+#endif
+
+#ifndef SOL_AAL
 #define SOL_AAL		265
+#endif
 
 
 #define HOSTS_ATM "/etc/hosts.atm"
@@ -88,6 +104,7 @@ int text2sap(const char *text,struct atm_sap *sap,int flags);
 int sap2text(char *buffer,int length,const struct atm_sap *sap,int flags);
 int sap_equal(const struct atm_sap *a,const struct atm_sap *b,int flags,...);
 
+int __t2q_get_rate(const char **text,int up);
 int __atmlib_fetch(const char **pos,...); /* internal use only */
 
 #endif
