@@ -65,6 +65,7 @@ static int do_svc(char *buffer,int length,const struct sockaddr_atmsvc *addr,
 {
     static int pure[] = { 20 };
     static int bin[] = { 1,2,10,6,1 };
+    static int local[] = { 1,12,6,1 };
     static int e164[] = { 4,6,1 };
 
     int orig_len,len,i,left,value;
@@ -94,9 +95,16 @@ static int do_svc(char *buffer,int length,const struct sockaddr_atmsvc *addr,
 	    switch (*addr->sas_addr.prv) {
 		case ATM_AFI_DCC:
 		case ATM_AFI_ICD:
+		case ATM_AFI_DCC_GROUP:
+		case ATM_AFI_ICD_GROUP:
 		    fmt = bin;
 		    break;
+		case ATM_AFI_LOCAL:
+		case ATM_AFI_LOCAL_GROUP:
+		    fmt = local;
+		    break;
 		case ATM_AFI_E164:
+		case ATM_AFI_E164_GROUP:
 		    for (i = 2; i < 17; i++)
 			if (addr->sas_addr.prv[i >> 1] & (0xf0 >> 4*(i & 1)))
 			    break;
