@@ -47,23 +47,29 @@ static void process(FIELD *start,int defines)
     int tmp,shift,next;
 
     for (walk = start; walk; walk = walk->next) {
+	walk->field = field; /* record it even if this isn't a real field (we
+				need to know the first field number later in
+				second.c and this way we don't have to search
+				for it) */
 	if (walk->structure) {
 	    int start_field,start_group;
+
 	    start_field = field;
 	    start_group = group;
-	    process(walk->structure->block,walk->structure->instances > 0);
-	    if (walk->structure->instances > 1)
+	    process(walk->my_block,walk->structure->instances > 0);
+	    if (walk->structure->instances > 1) {
 		walk->structure->instances = -1;
-	    walk->structure->fields = field-start_field;
-	    walk->structure->groups = group-start_group;
+		walk->structure->first_field = start_field;
+		walk->structure->fields = field-start_field;
+		walk->structure->groups = group-start_group;
+	    }
 	    continue;
 	}
-	/* set field number */
-	walk->seq = seq++;
+	walk->seq = seq++; /* number of fields, always increases */
 	if (*walk->id != '_') {
 	    if (defines) to_h("#define QF_%s %d\n",walk->id,field);
 	    to_test("    \"%s\",\n",walk->id);
-	    walk->field = field++;
+	    field++;
 	}
 	if (mask && walk->brk)
 	    die("mask 0x%02x at break (%s)\n",mask,walk->id);
@@ -168,7 +174,7 @@ void first(FIELD *def)
     def->group = group++;
     to_h("/*\n * Identifiers to access numbers of fields and of named\n");
     to_h(" * groups (unnamed groups don't need that). Field numbers are\n");
-    to_h(" * one-based. Group numbers are negative and -1-based.\n */\n\n");
+    to_h(" * zero-based. Group numbers are negative and -1-based.\n */\n\n");
     to_c("static unsigned char q_initial[Q_DATA_BYTES];\n\n");
     to_c("/*\n * Initialization of constant data. Could also do this in\n");
     to_c(" * the translator and output the resulting byte stream.\n */\n\n");
