@@ -1,8 +1,9 @@
 /* pdu.c - SSCOP (Q.2110) PDU reader */
 
-/* Written 1995-1997 by Werner Almesberger, EPFL-LRC */
+/* Written 1995-1998 by Werner Almesberger, EPFL-LRC/ICA */
 
 
+#include <stdint.h>
 #include <netinet/in.h> /* for ntohl */
 
 #include "pdu.h"
@@ -109,7 +110,7 @@ void print_pdu(const char *label,unsigned char type,void *data,
 int decompose_pdu(void *maa_arg,void *msg,int size,unsigned char *type,
   int *length,int *s,int *ps,int *r,int *mr,int *sq)
 {
-    __u32 *last;
+    uint32_t *last;
     unsigned char pad;
     int n;
 
@@ -123,7 +124,7 @@ int decompose_pdu(void *maa_arg,void *msg,int size,unsigned char *type,
 	if (pdu_maa) pdu_maa(maa_arg,'U',0);
 	return -1;
     }
-    last = (__u32 *) ((char *) msg+size-4);
+    last = (uint32_t *) ((char *) msg+size-4);
     *type = SSCOP_TYPE(*last);
     pad = SSCOP_PAD(*last);
     n = SSCOP_N(*last);
@@ -140,7 +141,7 @@ int decompose_pdu(void *maa_arg,void *msg,int size,unsigned char *type,
 		return -1;
 	    }
 	    *s = n;
-	    *ps = SSCOP_N(*(__u32 *) msg);
+	    *ps = SSCOP_N(*(uint32_t *) msg);
 	    break;
 	case SSCOP_STAT:
 	    if (size < 12) {
