@@ -1,6 +1,6 @@
 /* io.c - I/O operations */
  
-/* Written 1995-1998 by Werner Almesberger, EPFL-LRC/ICA */
+/* Written 1995-1999 by Werner Almesberger, EPFL-LRC/ICA */
  
 
 #include <stdlib.h>
@@ -31,11 +31,10 @@
 
 
 struct timeval now;
-struct sockaddr_atmpvc signaling_pvc;
 
 int stop = 0;
 
-static int kernel = -1,signaling;
+static int kernel = -1;
 static int need_connect = 0; /* non-zero if connection to kernel isn't
 				bi-directional yet */
 
@@ -118,7 +117,7 @@ static void close_kernel(void)
 /* ----- signaling interface ----------------------------------------------- */
 
 
-static int open_signaling(void)
+static int open_signaling(SIG_ENTITY *sig)
 {
     struct atm_qos qos;
     int s;
@@ -130,26 +129,27 @@ static int open_signaling(void)
     memset(&qos,0,sizeof(qos));
     qos.aal = ATM_AAL5;
     qos.rxtp.max_sdu = qos.txtp.max_sdu = MAX_Q_MSG;
-    if (sig_qos) {
-	if (text2qos(sig_qos,&qos,T2Q_DEFAULTS) < 0) {
-	    fprintf(stderr,"invalid qos: %s\n",sig_qos);
+    if (sig->sig_qos) {
+	if (text2qos(sig->sig_qos,&qos,T2Q_DEFAULTS) < 0) {
+	    fprintf(stderr,"invalid qos: %s\n",sig->sig_qos);
 	    return -1;
 	}
     }
     else {
-	if (sig_pcr == -1)
+	if (sig->sig_pcr == -1)
 	    qos.rxtp.traffic_class = qos.txtp.traffic_class = ATM_UBR;
 	else {
 	    qos.rxtp.traffic_class = qos.txtp.traffic_class = ATM_CBR;
-	    qos.rxtp.min_pcr = qos.txtp.min_pcr = sig_pcr;
+	    qos.rxtp.min_pcr = qos.txtp.min_pcr = sig->sig_pcr;
 	}
     }
     if (setsockopt(s,SOL_ATM,SO_ATMQOS,&qos,sizeof(qos)) < 0) {
 	perror("setsockopt SO_ATMQOS");
 	return -1;
     }
-    signaling_pvc.sap_family = AF_ATMPVC;
-    if (bind(s,(struct sockaddr *) &signaling_pvc,sizeof(signaling_pvc)) < 0) {
+    sig->signaling_pvc.sap_family = AF_ATMPVC;
+    if (bind(s,(struct sockaddr *) &sig->signaling_pvc,
+      sizeof(sig->signaling_pvc)) < 0) {
 	perror("bind");
 	return -1;
     }
@@ -157,32 +157,33 @@ static int open_signaling(void)
 }
 
 
-static void recv_signaling(void)
+static void recv_signaling(SIG_ENTITY *sig)
 {
     static unsigned char buffer[MAX_Q_MSG];
     int size;
 
-    size = read(signaling,buffer,MAX_Q_MSG);
+    size = read(sig->signaling,buffer,MAX_Q_MSG);
     if (size < 1) {
 	perror("read signaling");
 	return;
     }
-    diag(COMPONENT,DIAG_DEBUG,"FROM NET: %s PDU (%d bytes)",
-      pdu_name[size > 3 && !(size & 3) ? buffer[size-4] & 0xf : 0],size);
+    diag(COMPONENT,DIAG_DEBUG,"FROM NET (%d.%d.%d): %s PDU (%d bytes)",
+      S_PVC(sig),pdu_name[size > 3 && !(size & 3) ? buffer[size-4] & 0xf : 0],
+      size);
     if (debug) diag_dump(COMPONENT,DIAG_DEBUG,NULL,buffer,size);
-    from_net(buffer,size);
+    from_net(sig,buffer,size);
 }
 
 
-void to_net(void *msg,int size)
+void to_net(SIG_ENTITY *sig,void *msg,int size)
 {
     int wrote;
 
-    diag(COMPONENT,DIAG_DEBUG,"TO NET: %s PDU (%d bytes)",
+    diag(COMPONENT,DIAG_DEBUG,"TO NET (%d.%d.%d): %s PDU (%d bytes)",S_PVC(sig),
       pdu_name[size > 3 && !(size & 3) ? ((unsigned char *) msg)[size-4] & 0xf
       : 0],size);
     if (debug) diag_dump(COMPONENT,DIAG_DEBUG,NULL,msg,size);
-    wrote = write(signaling,msg,size);
+    wrote = write(sig->signaling,msg,size);
     if (wrote == size) return;
     if (wrote < 0) {
 	perror("signaling write");
@@ -193,47 +194,43 @@ void to_net(void *msg,int size)
 }
 
 
-static void close_signaling(void)
+static void close_signaling(SIG_ENTITY *sig)
 {
-    (void) close(signaling);
+    (void) close(sig->signaling);
 }
 
 
 /* ----- addresses --------------------------------------------------------- */
 
 
-LOCAL_ADDR local_addr[MAX_ADDRS+1];
-
-
-int get_addr(int itf)
+int get_addr(int itf,LOCAL_ADDR *local_addr)
 {
     struct atmif_sioc req;
-    struct sockaddr_atmsvc buffer[MAX_ADDRS];
+    struct sockaddr_atmsvc buffer[MAX_LOCAL_ADDRS];
     LOCAL_ADDR *from,*to;
     int addrs,i;
 
     for (from = to = local_addr; from->state != ls_unused; from++)
 	if (from->state != ls_removed) {
-	    from->state = itf == from->itf ? ls_removed : ls_same;
+	    from->state = ls_removed;
 	    *to++ = *from;
 	}
     req.number = itf;
     req.arg = buffer;
     req.length = sizeof(buffer);
-    if (ioctl(signaling,ATM_GETADDR,&req) < 0)
+    if (ioctl(entities->signaling,ATM_GETADDR,&req) < 0)
 	diag(COMPONENT,DIAG_FATAL,"ioctl ATM_GETADDR yields \"%s\"",
 	  strerror(errno));
     addrs = req.length/sizeof(struct sockaddr_atmsvc);
     for (i = 0; i < addrs; i++) {
 	for (from = local_addr; from->state != ls_unused; from++)
-	    if (from->itf == itf && atm_equal((struct sockaddr *) buffer+i,
+	    if (atm_equal((struct sockaddr *) (buffer+i),
 	      (struct sockaddr *) &from->addr,0,0)) break;
 	if (from->state != ls_unused) from->state = ls_same;
-	else if (to == local_addr+MAX_ADDRS-1)
+	else if (to == local_addr+MAX_LOCAL_ADDRS-1)
 		diag(COMPONENT,DIAG_WARN,"local address table overflow");
 	    else {
 		to->state = ls_added;
-		to->itf = itf;
 		to->addr = buffer[i];
 		to++;
 	    }
@@ -248,22 +245,29 @@ int get_addr(int itf)
 
 int open_all(void)
 {
+    SIG_ENTITY *sig,*purge;
+
     if (kernel == -1) kernel = open_kernel();
     if (kernel < 0) return -1;
-    signaling = open_signaling();
-    if (signaling < 0) {
-	close_kernel();
-	return -1;
+    for (sig = entities; sig; sig = sig->next) {
+	sig->signaling = open_signaling(sig);
+	if (sig->signaling < 0) {
+	    for (purge = entities; purge != sig; purge = purge->next)
+		close_signaling(purge);
+	    close_kernel();
+	    return -1;
+	}
     }
-    local_addr[0].state = ls_unused;
     return 0;
 }
 
 
 void close_all(void)
 {
+    SIG_ENTITY *sig;
+
     close_kernel();
-    close_signaling();
+    for (sig = entities;sig; sig = sig->next) close_signaling(sig);
 }
 
 
@@ -275,13 +279,17 @@ void init_current_time(void)
 
 void poll_loop(void)
 {
+    SIG_ENTITY *sig;
     fd_set perm,set;
     int fds,ret;
 
     FD_ZERO(&perm);
     FD_SET(kernel,&perm);
-    FD_SET(signaling,&perm);
-    fds = kernel > signaling ? kernel+1 : signaling+1;
+    fds = kernel+1;
+    for (sig = entities; sig; sig = sig->next) {
+	FD_SET(sig->signaling,&perm);
+	if (fds <= sig->signaling) fds = sig->signaling+1;
+    }
     gettimeofday(&now,NULL);
     while (!stop) {
 	set = perm;
@@ -301,7 +309,8 @@ void poll_loop(void)
 	    diag(COMPONENT,DIAG_DEBUG,"----------");
 	    gettimeofday(&now,NULL);
 	    if (FD_ISSET(kernel,&set)) recv_kernel();
-	    if (FD_ISSET(signaling,&set)) recv_signaling();
+	    for (sig = entities; sig; sig = sig->next)
+		if (FD_ISSET(sig->signaling,&set)) recv_signaling(sig);
 	    expire_timers();
 	      /* expire timers after handling messges to make sure we don't
 		 time out unnecessarily because of scheduling delays */
@@ -352,3 +361,18 @@ int get_pvc(int itf,int *vci)
     (void) close(s);
     return -error;
 }
+
+
+int get_link_rate(int itf)
+{
+    struct atmif_sioc req;
+    int rate;
+
+    req.number = itf;
+    req.arg = &rate;
+    req.length = sizeof(rate);
+    if (ioctl(entities->signaling,ATM_GETLINKRATE,&req) < 0)
+	diag(COMPONENT,DIAG_FATAL,"ioctl ATM_GETLINERATE yields \"%s\"",
+	  strerror(errno));
+    return rate;
+}
