@@ -6,6 +6,7 @@
 #include <stdio.h>
 #include <errno.h>
 #include <assert.h>
+#include <linux/atmdev.h>
 
 #include "atmd.h"
 
@@ -56,17 +57,16 @@ void on_T308_2(void *user) /* WAIT_REL or REL_REQ */
     diag(COMPONENT,DIAG_WARN,"Trouble: T308_2 has expired");
     if (sock->state != ss_wait_rel && sock->state != ss_rel_req)
 	complain(sock,"T308_2");
-    /* should initiate restart procedure now ... but we retry forver instead */
-    send_release(sock,ATM_CV_TIMER_EXP,308); /* @@@ ? */
     sock->conn_timer = NULL;
-    START_TIMER(sock,T308_2);
+    if (sock->state == ss_rel_req) send_close(sock);
+    free_sock(sock);
 }
 
 
 void on_T309(void *user)
 {
     diag(COMPONENT,DIAG_DEBUG,"T309 has expired");
-    clear_all_calls_on_T309();
+    clear_all_calls_on_T309(user);
 }
 
 
@@ -90,7 +90,7 @@ void on_T313(void *user) /* ACCEPTING */
 }
 
 
-#ifdef Q2963_1
+#if defined(Q2963_1) || defined(DYNAMIC_UNI)
 
 void on_T360(void *user)
 {
