@@ -9,14 +9,14 @@
 #include <string.h>
 #include <ctype.h>
 #include <sys/time.h>
-
-#include <linux/atm.h>
+#include <stdint.h> /* linux/atmsvc.h includes linux/atm.h */
 #include <linux/atmsvc.h>
 
 #include "atm.h"
 #include "atmd.h"
 #include "atmsap.h"
 #include "trace.h"
+#include "proto.h"
 
 
 #define DUMP_MODE
@@ -28,14 +28,16 @@ typedef struct _entry {
     struct timeval time;
     void (*print)(void *msg,int size);
     const char *comment;
+    struct sockaddr_atmpvc pvc; /* unused if !sap_family */
     void *msg;
     int size;
     struct _entry *next;
 } ENTRY;
 
 
-int trace_size = 0;
+int trace_size = DEFAULT_TRACE_SIZE;
 
+static struct sockaddr_atmpvc null_pvc;
 static int current_size = 0;
 static int sequence = 0;
 static ENTRY *first = NULL,*last = NULL;
@@ -207,7 +209,7 @@ static void print_kernel(void *msg,int size)
 
 
 static void store(void (*print)(void *msg,int size),const char *comment,
-  void *msg,int size)
+  const struct sockaddr_atmpvc *pvc,void *msg,int size)
 {
     ENTRY *entry;
 
@@ -216,6 +218,7 @@ static void store(void (*print)(void *msg,int size),const char *comment,
     entry->number = sequence++;
     entry->print = print;
     entry->comment = comment;
+    entry->pvc = *pvc;
     entry->msg = msg;
     entry->size = size;
     entry->next = NULL;
@@ -242,18 +245,19 @@ void trace_msg(const char *msg)
     if (!trace_size) return;
     buf = alloc(strlen(msg)+1);
     strcpy(buf,msg);
-    store(&print_text,"MESSAGE",buf,strlen(msg));
+    store(&print_text,"MESSAGE",&null_pvc,buf,strlen(msg));
 }
 
 
-void trace_uni(const char *comment,const void *msg,int size)
+void trace_uni(const char *comment,const SIG_ENTITY *sig,const void *msg,
+  int size)
 {
-    void *buf;
+    char *buf;
 
     if (!trace_size) return;
     buf = alloc(size);
     memcpy(buf,msg,size);
-    store(&print_uni,comment,buf,size);
+    store(&print_uni,comment,&sig->signaling_pvc,buf,size);
 }
 
 
@@ -264,7 +268,7 @@ void trace_kernel(const char *comment,const struct atmsvc_msg *msg)
     if (!trace_size) return;
     buf = alloc_t(struct atmsvc_msg);
     *buf = *msg;
-    store(&print_kernel,comment,buf,sizeof(*msg));
+    store(&print_kernel,comment,&null_pvc,buf,sizeof(*msg));
 }
 
 
@@ -277,8 +281,11 @@ char *get_trace(void)
 	string = NULL;
     }
     for (walk = first; walk; walk = walk->next) {
-	append("%6d (%d.%06d) %s:\n",walk->number,walk->time.tv_sec,
+	append("%6d (%d.%06d) %s",walk->number,walk->time.tv_sec,
 	  walk->time.tv_usec,walk->comment);
+	if (!walk->pvc.sap_family) append(":\n");
+	else append(" (%d.%d.%d):\n",walk->pvc.sap_addr.itf,
+	      walk->pvc.sap_addr.vpi,walk->pvc.sap_addr.vci);
 	new_line = 1;
 	walk->print(walk->msg,walk->size);
     }
